import java.util.Scanner;
import java.util.InputMismatchException;
import java.util.Random;
import java.util.Date;
/**
 * Simuleaza un bilet de loterie
 * 
 * @author Marius Joldos
 * @version 2009.10.16
 */
public class BiletLoterie
{
	/**
	 * invariant: cea mai mare valoare de numar care se poate extrage
	 * pentru orice obiect ExtragereLoterie
	 */
	public static final int VAL_MAX = Long.SIZE;
	/**
	 * invariant: valoarea cu care se imparte valoarea maxima extrasa
	 * pentru a stabili cite numere se pot extrage cel mult
	 */
	public static final int COTA_NUMERE = 8;
	private long nrPuse; // reprezentare pe biti a numerelor puse
	private int valMax = 49; // numarul maxim de extras
	private int nrNumere = 6; // cite numere se pot pune pe un bilet

	/**
	 * Constructor for objects of class BiletLoterie
	 * @param n cite numere se pot pune
	 * @param m valoarea celui mai mare numar care poate fi pus pe bilet
	 */
	public BiletLoterie(int n, int m)
	{
		boolean ok = false;
		if (m >= 1 && m <= VAL_MAX && m / COTA_NUMERE >= 1)
		{
			if (n >= 1 && n <= m / COTA_NUMERE)
			{
				nrNumere = n;
				valMax = m;
				ok = true;
			}
		}
		if (!ok)
			System.err
					.println("Argumentele au valori eronate. Valoarea maxima permisa= "
							+ m + " numarul de numere de pe bilet=" + n);

	}
	/**
	 * Returneaza o reprezentare ca String a continutului acestui bilet de loto
	 */
	public String toString()
	{
		String s = "Bilet pentru loterie " + nrNumere + " din " + valMax
				+ ".\nNumerele puse: ";
		for (int i = 0; i < valMax; i++)
			if ((nrPuse & (1L << i)) != 0)
				s += (i + 1) + " ";
		return s;
	}
	/**
	 * Tipareste continutul biletului loto
	 */
	public void listeaza()
	{
		System.out.println(this);
	}
	/**
	 * Accepta numere de scris pe bilet de la System.in
	 * si le scrie pe bilet
	 */
	public void acceptaNumere()
	{
		if (Long.bitCount(nrPuse) >= nrNumere)
			System.out.println("Bilet deja completat: " + this);
		System.out.print("Numere pe care le puneti pentru loteria " + nrNumere
				+ " din " + valMax + ": ");
		Scanner sc = new Scanner(System.in);
		int numarCurent = -1;
		boolean ok = true;
		String s;
		do
		{
			if (sc.hasNext("\\d+"))
			{
				s = sc.next("\\d+");
				System.err.print(s + " ");
				int val = Integer.parseInt(s);
				if (val >= 1 && val <= Long.SIZE)
				{
					nrPuse |= (1L << (val - 1));
					System.out
							.println("Numar acceptat: "
									+ val
									+ ((nrNumere - Long.bitCount(nrPuse) > 0) ? (". Mai trebuie "
											+ (nrNumere - Long.bitCount(nrPuse)) + " numere")
											: (". Toate numerele au fost introduse")));
				}
				else
					System.out.println("Numar nevalid:" + val);
			}
			else
			{
				if (sc.hasNext("\\D+"))
				{
					s = sc.next("\\D+");
					System.out.print(s + " se ignora\n");
				}
				else
					if (sc.hasNext("(\\d+\\D*)+|(\\D+\\d*)+"))
					{
						s = sc.next("(\\d+\\D*)+|(\\D+\\d*)+");
						System.out.print(s + " se ignora\n");
					}

			}
		}
		while (Long.bitCount(nrPuse) < nrNumere);
	}
	/** 
	 * returneaza un tablou cu numerele scrise pe acest bilet
	 * @return tablou de int cu numerele scrise pe bilet
	 */
	public int[] ceNumere()
	{
		int[] numere = new int[nrNumere];
		for (int i = 0, j = 0; i < valMax; i++)
		{
			if ((nrPuse & (1L << i)) != 0)
				numere[j++] = i + 1;
		}
		return numere;
	}
	 /**
     * Scrie biletul cu numere aleatoare
     */
    public void autoUmple()
    {
        Random gen = new Random(new Date().getTime());
        if (Long.bitCount(nrPuse) < nrNumere)
        do
        {
            nrPuse |= (1L << gen.nextInt(valMax));
        }
        while (Long.bitCount(nrPuse) < nrNumere);   
    }
}
