import java.util.Random;

/**
 * ExtragereLoterie se foloseste la simularea extragerii unui numar de numere dintr-un set dat
 * 
 * @author Marius Joldos
 * @version 2009.10.15
 */
public class ExtragereLoterie
{
    /**
     * invariant: cea mai mare valoare de numar care se poate extrage
     * pentru orice obiect ExtragereLoterie
     */
    public static final int VAL_MAX = Long.SIZE;
    /**
     * invariant: valoarea cu care se imparte valoarea maxima extrasa
     * pentru a stabili cite numere se pot extrage cel mult
     */
    public static final int COTA_NUMERE = 8;
    private static Random aleator = new Random(); // generator de numere aleatoare folosit la extragere
    private int valMax = 49; // reprezentare pe biti a numerelor extrase
    private int nrNumere = 6; // numarul maxim de extras
    private long nrExtrase = 0L; // cite numere se extrag


    /**
     * Constructor pentru obiecte de clasa ExtragereLoterie
     * construieste implicit loterie 6 din 49
     */
    public ExtragereLoterie()
    {
        this(6, 49);
    }
    /**
     * Constructor pentru obiecte de clasa ExtragereLoterie
     * @param n cite numere se extrag
     * @param m valoarea celui mai mare numar extras
     */
    public ExtragereLoterie(int n, int m)
    {
        boolean ok = false;
        if (m >= 1 && m <= VAL_MAX && m / COTA_NUMERE >= 1)
        {
            if (n >= 1 && n <= m / COTA_NUMERE)
            {
                nrNumere = n;
                valMax = m;
                ok = true;
            }
        }
        if (!ok)
            System.err
                    .println("Argumentele au valori eronate. Valoarea maxima extrasa= "
                            + m + " numarul de numere din extragere=" + n);
    }

    private void extrage1Numar()
    {
        nrExtrase |= (1L << aleator.nextInt(valMax));
    }
    /**
     * Executa o extragere
     */
    public void extrage()
    {
        if (Long.bitCount(nrExtrase) >= nrNumere)
            return;
        do
            extrage1Numar();
        while (Long.bitCount(nrExtrase) < nrNumere);
    }
    /**
     * Da o reprezentare ca sir a continutului acestei extrageri de loterie
     */
    public String toString()
    {
        String s = "ExtragereLoterie " + nrNumere + " din " + valMax
                + ".\nNumerele extrase: ";
        for (int i = 0; i < valMax; i++)
            if ((nrExtrase & (1L << i)) != 0)
                s += (i + 1) + " ";
        return s;
    }
    /**
     * Tipareste informatiile despre aceasta loterie
     */
    public void listeaza()
    {
        System.out.println(this);
    }
    /**
     * Verifica cite numere de pe biletul dat coincid cu cele
     * extrase. Scrie la stdout cele care coincid
     * @param b un obiect BiletLoterie de verificat
     */
    public void verificaBilet(BiletLoterie b)
    {
        int n = 0;
        int[] numere = b.ceNumere();
        if (numere.length != nrNumere)
        {
            System.out.println("Biletul nu este pentru acest tip de extragere ("
            + nrNumere + "!=" + numere.length + ")");
            return;
        }
        System.out.print("Numerele de pe bilet care au iesit sunt: ");
        for (int i = 0; i < numere.length; i++)
        {
            if ((nrExtrase & (1L << (numere[i] - 1))) != 0L)
            {
                System.out.print(numere[i] + " ");
                n++;
            }
        }
        if (n == 0)
            System.out.println(" none");
    }
}
